/************************************************************************************
managementofareamapirs
Copyright (c) 2010, International Rescue System Institute
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD license.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

//***********************************************************************************************//
//* map.cpp											*//
//***********************************************************************************************//


//-- [ヘッダファイル] ---------------------------------------------------------------------------//
#include "map.h"											// mapヘッダファイル読み込み
//-----------------------------------------------------------------------------------------------//


//-- [置換] -------------------------------------------------------------------------------------//
#define DEBUG_BMPFILEHEADER										// DEBUG定義 [デバッグ用]
#define DEBUG_BMPINFOHEADER										// DEBUG定義 [デバッグ用]
#define DEBUG_BMPIMAGEDATARAW										// DEBUG定義 [デバッグ用]
#define DEBUG_BMPIMAGEDATARGB										// DEBUG定義 [デバッグ用]
//-----------------------------------------------------------------------------------------------//


//-- [シグナル処理関数] -------------------------------------------------------------------------//
//- void signal_handler_IO(int status)								-//
//- 機能  ：UDP/IPからの受信完了時フラグ変更							-//
//- 引数  ：(int)  シグナル									-//
//- 戻り値：(void) なし										-//
//-----------------------------------------------------------------------------------------------//
void signal_handler_IO(int status)
{

	wait_flag = 0;											// フラグリセット

	return;												// 戻り値

}
//-----------------------------------------------------------------------------------------------//


//-- [時間確認関数] -----------------------------------------------------------------------------//
//- double gettimeofday_sec(void)								-//
//- 機能  ：現在時刻の取得を行う								-//
//- 引数  ：(void)   なし									-//
//- 戻り値：(double) 現在時刻 [s]								-//
//-----------------------------------------------------------------------------------------------//
double gettimeofday_sec(void)
{


	//-- [変数宣言] -------------------------------------------------------------------------//
	//-											-//
	//- struct time{									-//
	//-	time_t tv_sec;			- 秒						-//
	//-	suseconds_t tv_usec;		- マイクロ秒					-//
	//- }											-//
	//-											-//
	//---------------------------------------------------------------------------------------//
	struct timeval tv;										// 使用されたユーザ時間格納構造体

	gettimeofday(&tv, NULL);									// 時刻取得

	return tv.tv_sec + (double)tv.tv_usec * 1e-6;							// 戻り値 [現在時刻]

}
//-----------------------------------------------------------------------------------------------//


//-- [画像構造体メモリ確保関数] -----------------------------------------------------------------//
//- Picture* newPicture(void)									-//
//- 機能  ：画像データの構造体メモリの確保を行う						-//
//- 引数  ：(void)     なし									-//
//- 戻り値：(Picture*) 画像データ構造体のポインタ						-//
//-----------------------------------------------------------------------------------------------//
Picture* newpicture(void)
{


	//-- [変数宣言] -------------------------------------------------------------------------//

	Picture *ppic;											// 画像データ格納構造体ポインタ

	//---------------------------------------------------------------------------------------//


	//-- [メモリ確保] -----------------------------------------------------------------------//

	ppic = (Picture*)malloc(sizeof(Picture));							// メモリ確保
	if(ppic == NULL) return NULL;									// メモリ確保失敗

	//---------------------------------------------------------------------------------------//


	//-- [初期化] ---------------------------------------------------------------------------//

	ppic->x = 0;											// x軸画素数初期化
	ppic->y = 0;											// y軸画素数初期化
	ppic->r = NULL;											// RED要素初期化
	ppic->g = NULL;											// GREEN要素初期化
	ppic->b = NULL;											// BLUE要素初期化
	ppic->a = NULL;											// ALPHA要素初期化

	//---------------------------------------------------------------------------------------//


	return ppic;											// 戻り値 [正常終了]

}
//-----------------------------------------------------------------------------------------------//


//-- [画像構造体メモリ解放関数] -----------------------------------------------------------------//
//- void deletepicture(Picture* ppic)								-//
//- 機能  ：画像データの構造体メモリの解放を行う						-//
//- 引数  ：(Picture*) 画像データ構造体のポインタ						-//
//- 戻り値：(void)     なし									-//
//-----------------------------------------------------------------------------------------------//
void deletepicture(Picture* ppic)
{


	//-- [メモリ解放] -----------------------------------------------------------------------//

	free(ppic->r);												// RED要素メモリ解放
	free(ppic->g);												// GREEN要素メモリ解放
	free(ppic->b); 												// BLUE要素メモリ解放
	free(ppic->a); 												// ALPHA要素メモリ解放
	ppic = NULL;												// 画像データ構造体リセット

	//---------------------------------------------------------------------------------------//


	return;												// 戻り値 [正常終了]

}
//-----------------------------------------------------------------------------------------------//


//-- [bitmap取得関数] ---------------------------------------------------------------------------//
//- Picture* getbitmap(FILE* fd)								-//
//- 機能  ：bitmapデータの取得を行う								-//
//- 引数  ：(FILE*)    bitmapのファイルディスクリプタ						-//
//- 戻り値：(Picture*) 画像データ格納構造体ポインタ						-//
//- 注意  : getbitmap関数使用時はdeletepicture関数によりメモリを解放すること			-//
//-----------------------------------------------------------------------------------------------//
Picture* getbitmap(FILE* fd)
{


	//-- [変数宣言] -------------------------------------------------------------------------//

	int i;												// カウンタ格納変数
	int j;												// カウンタ格納変数
	int x;												// x軸画素数格納変数
	int y;												// y軸画像数格納変数
	int imagesize;											// 画像サイズ格納変数
	int pad;											// ゼロパディング数格納変数
	unsigned char *buf_top;										// 先頭画像データ格納変数ポインタ
	unsigned char *buf;										// 画像データ格納変数ポインタ
	Picture *ppic;											// 画像データ格納構造体ポインタ
	BMPFILEHEADER bf;										// ファイルヘッダ格納構造体
	BMPINFOHEADER bi;										// 情報ヘッダ格納構造体

	//---------------------------------------------------------------------------------------//


	//-- [引数チェック] ---------------------------------------------------------------------//

	if(fd == NULL) return NULL;									// 引数チェック

	//---------------------------------------------------------------------------------------//


	//-- [ファイルヘッダ処理] ---------------------------------------------------------------//

	if(fread((void*)&bf, sizeof(BMPFILEHEADER), 1, fd) != 1) return NULL;				// ファイルヘッダ読み込み

	if(bf.bfType != 0x4d42) return NULL;								// ファイルタイプチェック [BM]
	if(bf.bfReserved1 != 0) return NULL;								// 予約領域1チェック [0]
	if(bf.bfReserved2 != 0) return NULL;								// 予約領域2チェック [0]
	if(bf.bfOffBits != 54) return NULL;								// オフセットサイズチェック [54 byte]


	//-- [デバッグbmpfileheader] ------------------------------------------------------------//

	#ifndef DEBUG_BMPFILEHEADER									// デバッグ表示開始

	printf("\n[bmpfileheader]\n");									// label
	printf("bf.bfType      = %c%c\n",
		(char)(bf.bfType & 0x00ff), (char)((bf.bfType >> 8) & 0x00ff));				// bf.bfType
	printf("bf.bfSize      = %ld\n", bf.bfSize);							// bf.bfSize
	printf("bf.bfReserved1 = %d\n", bf.bfReserved1);						// bf.bfReserved1
	printf("bf.bfReserved2 = %d\n", bf.bfReserved2);						// bf.bfReserved2
	printf("bf.bfOffBits   = %ld\n\n", bf.bfOffBits);						// bf.bfOffBits
	fflush(stdout);											// バッファフラッシュ [printf表示用]

	#endif												// デバッグ表示終了

	//---------------------------------------------------------------------------------------//


	//---------------------------------------------------------------------------------------//


	//-- [情報ヘッダ処理] -------------------------------------------------------------------//

	if(fread((void*)&bi, sizeof(BMPINFOHEADER), 1, fd) != 1) return NULL;				// 情報ヘッダ読み込み

	if(bi.biSize != 40) return NULL;								// 情報ヘッダサイズチェック [40 byte]
	if(bi.biPlanes != 1) return NULL;								// プレーン数チェック [1]
	if(bi.biBitCount != 24) return NULL;								// 色ビット数チェック [24]
	if(bi.biCompression != 0) return NULL;								// 圧縮形式チェック [0]
	if(bi.biClrUsed != 0) return NULL;								// 格納パレット数チェック [0]
	if(bi.biClrImportant != 0) return NULL;								// 重要色数チェック [0]

	imagesize = bi.biSizeImage;									// 画像サイズ格納
	x = bi.biWidth;											// 画像幅格納
	y = bi.biHeight;										// 画像高さ格納


	//-- [デバッグbmpinfoheader] ------------------------------------------------------------//

	#ifndef DEBUG_BMPINFOHEADER									// デバッグ表示開始

	printf("\n[bmpinfoheader]\n");									// label
	printf("bi.biSize          = %ld\n", bi.biSize);						// bi.biSize
	printf("bi.biWidth         = %ld\n", bi.biWidth);						// bi.biWidth
	printf("bi.biHeight        = %ld\n", bi.biHeight);						// bi.biHeight
	printf("bi.biPlanes        = %d\n", bi.biPlanes);						// bi.biPlanes
	printf("bi.biBitCount      = %d\n", bi.biBitCount);						// bi.biBitCount
	printf("bi.biCompression   = %ld\n", bi.biCompression);						// bi.biCompression
	printf("bi.biSizeImage     = %ld\n", bi.biSizeImage);						// bi.biSizeImage
	printf("bi.biXPelsPerMeter = %ld\n", bi.biXPelsPerMeter);					// bi.biXPelsPerMeter
	printf("bi.biYPelsPerMeter = %ld\n", bi.biYPelsPerMeter);					// bi.biYPelsPerMeter
	printf("bi.biClrUsed       = %ld\n", bi.biClrUsed);						// bi.biClrUsed
	printf("bi.biClrImportant  = %ld\n\n", bi.biClrImportant);					// bi.biClrImportant
	fflush(stdout);											// バッファフラッシュ [printf表示用]

	#endif												// デバッグ表示終了

	//---------------------------------------------------------------------------------------//


	//---------------------------------------------------------------------------------------//


	//-- [画像データ処理] -------------------------------------------------------------------//

	buf_top = buf = (unsigned char*)malloc(imagesize);						// メモリ確保
	if(buf == NULL) return NULL;									// メモリ確保失敗

	if(fread((void*)buf, 1, imagesize, fd) != (unsigned int)imagesize){				// 画像データ取得

		free(buf_top);										// メモリ解放
		return NULL;										// 戻り値 [異常終了]

	}


	//-- [デバッグbmpimagedataraw] ----------------------------------------------------------//

	#ifndef DEBUG_BMPIMAGEDATARAW									// デバッグ表示開始

	printf("\n[bmpimagedataraw]\n");								// label

	for(i = 0; i < imagesize; i++){

		if(*(buf + i) < 0x10) printf("0%x ", *(buf + i));					// buf
		else printf("%x ", *(buf + i));								// buf

		if(i != 0){

			if(((i + 1) % 0x10) == 0) printf("\n");						// 改行
			else if(((i + 1) % 0x08) == 0) printf(" ");					// 空白

		}

	}

	printf("\n");											// 改行
	fflush(stdout);											// バッファフラッシュ [printf表示用]

	#endif												// デバッグ表示終了

	//---------------------------------------------------------------------------------------//


	ppic = newpicture();										// 画像データ構造体メモリ確保
	if(ppic == NULL){

		free(buf_top);										// メモリ解放
		return NULL;										// 戻り値 [異常終了]

	}

	ppic->x = x;											// x軸画素数格納
	ppic->y = y;											// y軸画素数格納
	ppic->r = (unsigned char *)malloc(sizeof(unsigned char) * x * y);				// RED要素メモリ確保
	ppic->g = (unsigned char *)malloc(sizeof(unsigned char) * x * y);				// GREEN要素メモリ確保
	ppic->b = (unsigned char *)malloc(sizeof(unsigned char) * x * y);				// BLUE要素メモリ確保
	if((ppic->r == NULL) || (ppic->g == NULL) || (ppic->b == NULL)){

		deletepicture(ppic);									// 画像データ構造体メモリ解放
		free(buf_top);										// メモリ解放
		return NULL;										// 戻り値 [異常終了]

	}

	pad = 4 - ((3 * bi.biWidth) % 4);								// ゼロパディング算出
	if(pad == 4) pad = 0;										// ゼロパディング処理 [0]

	for(i = 1 - y; i < 1; i++){									// y軸移動 [逆方向]

		for(j = 0; j < x; j++){									// x軸移動 [正方向]

			ppic->b[j - i * x] = *(buf++);							// BLUE要素格納
			ppic->g[j - i * x] = *(buf++);							// GREEN要素格納
			ppic->r[j - i * x] = *(buf++);							// RED要素格納
		}

		buf = buf + pad;									// ゼロパディングシフト

	}


	//-- [デバッグbmpimagedatargb] ----------------------------------------------------------//

	#ifndef DEBUG_BMPIMAGEDATARGB									// デバッグ表示開始

	printf("\n[bmpimagedatargb]\n");								// label

	for(i = 0; i < y; i++){

		for(j = 0; j < x; j++){

			if(ppic->r[j + i * x] < 0x10) printf("0%x", ppic->r[j + i * x]);		// r
			else printf("%x", ppic->r[j + i * x]);						// r
			if(ppic->g[j + i * x] < 0x10) printf("0%x", ppic->g[j + i * x]);		// g
			else printf("%x", ppic->g[j + i * x]);						// g
			if(ppic->b[j + i * x] < 0x10) printf("0%x", ppic->b[j + i * x]);		// b
			else printf("%x", ppic->b[j + i * x]);						// b

		}

		printf("\n");										// 改行

	}

	printf("\n");											// 改行
	fflush(stdout);											// バッファフラッシュ [printf表示用]

	#endif												// デバッグ表示終了

	//---------------------------------------------------------------------------------------//


	free(buf_top);											// メモリ解放

	//---------------------------------------------------------------------------------------//


	return ppic;											// 戻り値 [正常終了]

}
//-----------------------------------------------------------------------------------------------//
